require 'iconv'
require 'nkf'
require 'set'

class CardController < ApplicationController
  def list
    @cards = Card.all_sorted
    @html_title = _('Card List')
  end
  def show
    @card = Card.find(params[:id])
    @html_title = _('Card Detail: %{full_name}') % {:full_name => @card.full_name}
  end
  def do_new
    @languages = Language.all_sorted
    @html_title = _('New Card')
  end
  def new
    @card = Card.new
    @card.language = Language.find(:first, :conditions => {:name => GetText.locale.language})
    do_new
  end
  def create
    @card = Card.new(params[:card])
    if @card.save
      redirect_to :action => 'list'
    else
      do_new
      render :action => 'new'
    end
  end
  def do_edit
    @languages = Language.all_sorted
    @html_title = _('Edit Card: %{full_name}') % {:full_name => @card.full_name}
  end
  def edit
    @card = Card.find(params[:id])
    do_edit
  end
  def update
    @card = Card.find(params[:id])
    if @card.update_attributes(params[:card])
      redirect_to :action => 'show', :id => @card.id
    else
      do_edit
      render :action => 'edit'
    end
  end
  def delete
    Card.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
  def export
    card = Card.find(params[:id])
    format = params[:format] || 'vcf'
    do_download(format, '%s.%s' % [card.full_name, format], [card])
  end
  def export_all
    format = params[:format] || 'vcf'
    do_download(format, '%s.%s' % ['all', format], Card.all_sorted)
  end
  def do_download(format, filename, cards)
    case format
    when 'vcf'
      output = ''
      cards.each { |card|
        vcard = card.to_vcard
        output << vcard.encode(0)
      }
      charset = params[:charset] || 'UTF-8'
      if charset.downcase != 'UTF-8'
        output = Iconv.conv(charset, 'UTF-8', output)
      end
      send_data output,
        :disposition => 'inline',
        :type => 'text/x-vcard; charset=' + charset,
        :filename => filename
    else
      raise _('%{format}: Unsupported format.') % {:format => format}
    end
  rescue => e
    render :text => e.to_s,
      :type => 'text/plain; charset=UTF-8',
      :status => '406 Not Acceptable'
  end
  def import_form
    @html_title = _('Import Cards')
    render :action => 'import_form'
  end
  def import
    @html_title = _('Import Cards')
    @imported = []
    @failed = []
    input = params[:import][:file].read
    input = NKF.nkf('-wxm0', input)
    Vpim::Vcard.decode(input).each { |vcard|
      card = Card.new_from_vcard(vcard)
      if card.save
        @imported << card
      else
        @failed << card
      end
    }
    raise if !@failed.empty?
    render :action => 'import_form'
  rescue => e
    render :action => 'import_form',
      :status => '406 Not Acceptable'
  end
end
